"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getQuery = exports.toParams = exports.xhr_put = exports.xhr_post = exports.xhr_del = exports.xhr_get = void 0;
/**
 * 默认的请求配置
 */
var DEFAULT_XHR_CFG = {
    timeout: 3000,
    withCredentials: false,
    parseContentType: 'json'
};
/**
 *
 * @param getOrDel
 * @param url
 * @param cb
 * @param params
 * @param cfg
 */
function getOrDel(getOrDel, url, cb, params, cfg) {
    if (cfg === void 0) { cfg = DEFAULT_XHR_CFG; }
    var xhr = initXhr(cfg);
    if (params != null)
        url += '?' + toParams(params);
    xhr.open(getOrDel.toUpperCase(), url, true);
    xhr.onreadystatechange = function () {
        responseHandle(this, cb, cfg);
    };
    xhr.send();
}
/**
 *
 * @param method
 * @param url
 * @param cb
 * @param params
 * @param cfg
 */
function postOrPut(method, url, cb, params, cfg) {
    if (cfg === void 0) { cfg = DEFAULT_XHR_CFG; }
    var xhr = initXhr(cfg);
    xhr.open(method, url, true);
    xhr.onreadystatechange = function () {
        responseHandle(this, cb, cfg);
    };
    // 此方法必须在 open()方法和 send() 之间调用
    if (!cfg.contentType) // 如未设置，默认为表单请求
        xhr.setRequestHeader("Content-Type", "application/x-www-form-urlencoded");
    var _params = typeof params != 'string' ? toParams(params) : params;
    if (_params)
        xhr.send(_params);
    else
        xhr.send();
}
/**
 * XHR GET 请求
 *
 * @param url       请求地址
 * @param cb        回调函数 @example (json: {}, text: string) => void;
 * @param params    参数，必填，如无填空字符串 ""；参数类型是json；参数值会进行 URL 编码，最后附加到 QueryString 中
 * @param cfg       配置，可选的
 */
function xhr_get(url, cb, params, cfg) {
    if (cfg === void 0) { cfg = DEFAULT_XHR_CFG; }
    getOrDel('get', url, cb, params, cfg);
}
exports.xhr_get = xhr_get;
/**
 * XHR DELETE 请求
 *
 * @param url       请求地址
 * @param cb        回调函数 @example (json: {}, text: string) => void;
 * @param params    参数，必填，如无填空字符串 ""；参数类型是json；参数值会进行 URL 编码，最后附加到 QueryString 中
 * @param cfg       配置，可选的
 */
function xhr_del(url, cb, params, cfg) {
    if (cfg === void 0) { cfg = DEFAULT_XHR_CFG; }
    getOrDel('delete', url, cb, params, cfg);
}
exports.xhr_del = xhr_del;
/**
 * XHR POST 请求
 *
 * @param url       请求地址
 * @param cb        回调函数 @example (json: {}, text: string) => void;
 * @param params    参数，必填，如无填空字符串 ""；参数类型可以是字符串或 json；参数值会进行 URL 编码
 * @param cfg       配置，可选的
 */
function xhr_post(url, cb, params, cfg) {
    if (cfg === void 0) { cfg = DEFAULT_XHR_CFG; }
    postOrPut('post', url, cb, params, cfg);
}
exports.xhr_post = xhr_post;
/**
 * XHR PUT 请求
 *
 * @param url       请求地址
 * @param cb        回调函数 @example (json: {}, text: string) => void;
 * @param params    参数，必填，如无填空字符串 ""；参数类型可以是字符串或 json；参数值会进行 URL 编码
 * @param cfg       配置，可选的
 */
function xhr_put(url, cb, params, cfg) {
    if (cfg === void 0) { cfg = DEFAULT_XHR_CFG; }
    postOrPut('put', url, cb, params, cfg);
}
exports.xhr_put = xhr_put;
/**
 * 初始化 XHR
 *
 * @param cfg
 * @returns
 */
function initXhr(cfg) {
    var xhr = new XMLHttpRequest();
    if (cfg && cfg.timeout) {
        xhr.timeout = cfg.timeout;
        xhr.ontimeout = function (e) { alert('系统异常，XHR 连接服务超时'); };
    }
    if (cfg && cfg.withCredentials)
        xhr.withCredentials = true;
    return xhr;
}
/**
 * 错误处理
 *
 * @param xhr
 */
function errHandle(xhr) {
    var msg;
    if (xhr.status <= 400)
        msg = '请求参数错误或者权限不足。';
    else if (xhr.status <= 500)
        msg = '服务端异常。';
    else
        msg = "\u672A\u77E5\u5F02\u5E38\uFF0CHTTP code:" + xhr.status + "\u3002";
    if (!xhr.responseText)
        msg += " 服务端返回空的字符串！";
    console.error(msg, xhr.responseText);
    alert(msg);
}
/**
 * 响应处理
 *
 * @param xhr
 * @param cb
 * @param cfg
 */
function responseHandle(xhr, cb, cfg) {
    if (xhr.readyState == 4) {
        if ((xhr.status >= 200 && xhr.status < 300) || xhr.status == 304) {
            var text = xhr.responseText;
            var json = void 0;
            if (!text)
                console.warn('服务端没有返回任何字符串');
            switch (cfg.parseContentType) {
                case 'text':
                    break;
                case 'xml':
                    json = xhr.responseXML;
                    break;
                case 'json':
                default:
                    try {
                        json = JSON.parse(text);
                    }
                    catch (e) {
                        console.warn(e);
                        alert('解析 JSON 时候发生错误，非法 JSON');
                    }
            }
            cb && cb(json, text);
        }
        else
            errHandle(xhr);
    }
}
/**
 * 对象转换为 URL 参数列表，用 & 分隔
 *
 * @param {Object} param JSON 对象
 * @returns URL 参数列表
 */
function toParams(param) {
    var result = "";
    for (var name_1 in param) {
        if (typeof param[name_1] != "function")
            result += "&" + name_1 + "=" + encodeURIComponent(param[name_1]);
    }
    return result.substring(1);
}
exports.toParams = toParams;
/**
 * 获取 QueryString 的某个参数
 *
 * @param val
 * @returns
*/
function getQuery(val) {
    var w = location.hash.indexOf('?');
    var query = location.hash.substring(w + 1);
    var vars = query.split('&');
    for (var i = 0; i < vars.length; i++) {
        var pair = vars[i].split('=');
        if (pair[0] == val)
            return pair[1];
    }
    return '';
}
exports.getQuery = getQuery;
//# sourceMappingURL=xhr.js.map